# Copyright (c) 2015-2017 Claudiu Popa <pcmanticore@gmail.com>
# Copyright (c) 2017 Derek Gustafson <degustaf@gmail.com>

# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/PyCQA/pylint/blob/master/COPYING

"""Checker for anything related to the async protocol (PEP 492)."""

import sys

import astroid
from astroid import exceptions

from pylint import checkers
from pylint.checkers import utils as checker_utils
from pylint import interfaces
from pylint import utils


class AsyncChecker(checkers.BaseChecker):
    __implements__ = interfaces.IAstroidChecker
    name = 'async'
    msgs = {
        'E1700': ('Yield inside async function',
                  'yield-inside-async-function',
                  'Used when an `yield` or `yield from` statement is '
                  'found inside an async function.',
                  {'minversion': (3, 5)}),
        'E1701': ("Async context manager '%s' doesn't implement __aenter__ and __aexit__.",
                  'not-async-context-manager',
                  'Used when an async context manager is used with an object '
                  'that does not implement the async context management protocol.',
                  {'minversion': (3, 5)}),
    }

    def open(self):
        self._ignore_mixin_members = utils.get_global_option(self, 'ignore-mixin-members')

    @checker_utils.check_messages('yield-inside-async-function')
    def visit_asyncfunctiondef(self, node):
        for child in node.nodes_of_class(astroid.Yield):
            if child.scope() is node and (sys.version_info[:2] == (3, 5) or
                                          isinstance(child, astroid.YieldFrom)):
                self.add_message('yield-inside-async-function', node=child)

    @checker_utils.check_messages('not-async-context-manager')
    def visit_asyncwith(self, node):
        for ctx_mgr, _ in node.items:
            infered = checker_utils.safe_infer(ctx_mgr)
            if infered is None or infered is astroid.YES:
                continue

            if isinstance(infered, astroid.Instance):
                try:
                    infered.getattr('__aenter__')
                    infered.getattr('__aexit__')
                except exceptions.NotFoundError:
                    if isinstance(infered, astroid.Instance):
                        # If we do not know the bases of this class,
                        # just skip it.
                        if not checker_utils.has_known_bases(infered):
                            continue
                        # Just ignore mixin classes.
                        if self._ignore_mixin_members:
                            if infered.name[-5:].lower() == 'mixin':
                                continue
                else:
                    continue

            self.add_message('not-async-context-manager',
                             node=node, args=(infered.name, ))


def register(linter):
    """required method to auto register this checker"""
    linter.register_checker(AsyncChecker(linter))



"""Checks use of "too-complex" check"""


def f1():
    """McCabe rating: 1"""
    pass


def f2(n):
    """McCabe rating: 1"""
    k = n + 4
    s = k + n
    return s


def f3(n):
    """McCabe rating: 3"""
    if n > 3:
        return "bigger than three"
    elif n > 4:
        return "is never executed"
    else:
        return "smaller than or equal to three"


def f4():
    """McCabe rating: 2"""
    for i in range(10):
        print(i)


def f5(mylist):
    """McCabe rating: 2"""
    for i in mylist:
        print(i)
    else:
        print(None)


def f6(n):
    """McCabe rating: 2"""
    if n > 4:
        return f(n - 1)
    else:
        return n


def f7():
    """McCabe rating: 3"""
    def b():
        """McCabe rating: 2"""
        def c():
            """McCabe rating: 1"""
            pass
        c()
    b()


def f8():
    """McCabe rating: 4"""
    try:
        print(1)
    except TypeA:
        print(2)
    except TypeB:
        print(3)
    else:
        print(4)


def f9():
    """McCabe rating: 9"""
    myint = 2
    if myint > 5:
        pass
    else:
        if myint <= 5:
            pass
        else:
            myint = 3
            if myint > 2:
                if myint > 3:
                    pass
                elif myint == 3:
                    pass
                elif myint < 3:
                    pass
                else:
                    if myint:
                        pass
            else:
                if myint:
                    pass
                myint = 4


def f10():
    """McCabe rating: 11"""
    myint = 2
    if myint == 5:
        return myint
    elif myint == 6:
        return myint
    elif myint == 7:
        return myint
    elif myint == 8:
        return myint
    elif myint == 9:
        return myint
    elif myint == 10:
        if myint == 8:
            while True:
                return True
        elif myint == 8:
            with myint:
                return 8
    else:
        if myint == 2:
            return myint
        return myint
    return myint


class MyClass1(object):
    """Class of example to test mccabe"""
    _name = 'MyClass'  # To force a tail.node=None

    def method1():
        """McCabe rating: 1"""
        pass

    def method2(self, param1):
        """McCabe rating: 18"""
        if not param1:
            pass
        pass
        if param1:
            pass
        else:
            pass

        pass

        if param1:
            pass
        if param1:
            pass
        if param1:
            pass
        if param1:
            pass
        if param1:
            pass
        if param1:
            pass
        if param1:
            for value in range(5):
                pass

        pass
        for count in range(6):
            with open('myfile') as fp:
                count += 1
            pass
        pass
        try:
            pass
            if not param1:
                pass
            else:
                pass
            if param1:
                raise BaseException('Error')
            with open('myfile2') as fp2:
                pass
            pass
        finally:
            if param1 is not None:
                pass
            for count2 in range(8):
                try:
                    pass
                except BaseException('Error2'):
                    pass
        return param1


for count in range(10):
    if count == 1:
        exit(0)
    elif count == 2:
        exit(1)
    else:
        exit(2)


def method3(self):
    try:
        if True:
            pass
        else:
            pass
    finally:
        pass
    return True



# pylint: disable=old-style-class,missing-docstring,too-few-public-methods
__version__ = "1.0"
SOME_CONSTANT = 42  # [invalid-name]


def sayHello(someArgument):
    return [someArgument * someValue for someValue in range(10)]


class MyClass:  # [invalid-name]
    def __init__(self, argX):
        self._mySecretX = argX

    @property
    def myPublicX(self):
        return self._mySecretX * 2

    def __eq__(self, other):
        return isinstance(other, MyClass) and self.myPublicX == other.myPublicX


def say_hello():  # [invalid-name]
    pass



# pylint: disable=missing-docstring
def func():
    """Test that a variable defined in a finally clause does not trigger a false positive"""
    try:
        variable = 1
        yield variable
    finally:
        variable = 2
        yield variable



# pylint: disable=missing-docstring, bad-whitespace
"""Regression test for https://github.com/PyCQA/pylint/issues/1168

The problem was that we weren't handling keyword-only arguments annotations,
which means we were never processing them.
"""


from typing import Optional, Callable, Iterable


def func1(arg: Optional[Callable]=None):
    return arg


def func2(*, arg: Optional[Iterable]=None):
    return arg



# pylint: disable=literal-comparison,missing-docstring,misplaced-comparison-constant

X = 123
Y = len('test')

if X is 0:  # [compare-to-zero]
    pass

if Y is not 0:  # [compare-to-zero]
    pass

if X == 0:  # [compare-to-zero]
    pass

if Y != 0:  # [compare-to-zero]
    pass

if X > 0:  # [compare-to-zero]
    pass

if X < 0: #  this is allowed
    pass

if 0 < X:  # [compare-to-zero]
    pass

if 0 > X: #  this is allowed
    pass



# pylint: disable=missing-docstring, too-few-public-methods, import-error,unused-argument

import abc

import six
from unknown import Unknown


class InvalidAsMetaclass(object):
    pass


class ValidAsMetaclass(type):
    pass


@six.add_metaclass(type)
class FirstGood(object):
    pass


@six.add_metaclass(abc.ABCMeta)
class SecondGood(object):
    pass


@six.add_metaclass(Unknown)
class ThirdGood(object):
    pass


@six.add_metaclass(ValidAsMetaclass)
class FourthGood(object):
    pass



"""Test that disabling too-many-lines on any line works."""



# pylint: disable=too-many-lines
__revision__ = 0




































































































































































































































































































ZERFZAER = 3














































































































































































































































































































































































































































































































































































































































































































































HEHEHE = 2



# Copyright (c) 2016-2017 Claudiu Popa <pcmanticore@gmail.com>
# Copyright (c) 2016-2017 Derek Gustafson <degustaf@gmail.com>

# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/PyCQA/pylint/blob/master/COPYING

"""Tests for the pylint checker in :mod:`pylint.extensions.check_elif
"""

import os.path as osp

import pytest

from pylint.extensions.redefined_variable_type import MultipleTypesChecker
from pylint.lint import fix_import_path


EXPECTED = [
    'Redefinition of self.var1 type from int to float',
    'Redefinition of var type from int to str',
    'Redefinition of myint type from int to bool',
    'Redefinition of _OK type from bool to str',
    'Redefinition of instance type from redefined.MyClass to bool',
    'Redefinition of SOME_FLOAT type from float to int',
    'Redefinition of var3 type from str to int',
    'Redefinition of var type from bool to int',
    'Redefinition of var4 type from float to str',
]


@pytest.fixture(scope="module")
def checker(checker):
    return MultipleTypesChecker


@pytest.fixture(scope="module")
def disable(disable):
    return ['I']


def test_types_redefined(linter):
    elif_test = osp.join(osp.dirname(osp.abspath(__file__)), 'data',
                         'redefined.py')
    with fix_import_path([elif_test]):
        linter.check([elif_test])
    msgs = sorted(linter.reporter.messages, key=lambda item: item.line)
    assert len(msgs) == 9
    for msg, expected in zip(msgs, EXPECTED):
        assert msg.symbol == 'redefined-variable-type'
        assert msg.msg == expected



"""Test for old tenrary constructs"""
from UNINFERABLE import condition, true_value, false_value, some_callable  # pylint: disable=import-error

SOME_VALUE1 = true_value if condition else false_value
SOME_VALUE2 = condition and true_value or false_value  # [consider-using-ternary]
SOME_VALUE3 = (false_value, true_value)[condition]  # [consider-using-ternary]


def func1():
    """Ternary return value correct"""
    return true_value if condition else false_value


def func2():
    """Ternary return value incorrect"""
    return condition and true_value or false_value  # [consider-using-ternary]


def func3():
    """Ternary return value incorrect"""
    return (false_value, true_value)[condition]  # [consider-using-ternary]


SOME_VALUE4 = some_callable(condition) and 'ERROR' or 'SUCCESS'  # [consider-using-ternary]
SOME_VALUE5 = SOME_VALUE1 > 3 and 'greater' or 'not greater'  # [consider-using-ternary]
SOME_VALUE6 = (SOME_VALUE2 > 4 and SOME_VALUE3) and 'both' or 'not'  # [consider-using-ternary]
SOME_VALUE7 = 'both' if (SOME_VALUE2 > 4) and (SOME_VALUE3) else 'not'
SOME_VALUE8 = SOME_VALUE1 and SOME_VALUE2 and SOME_VALUE3 or SOME_VALUE4
SOME_VALUE9 = SOME_VALUE1 and False or SOME_VALUE2  # [simplify-boolean-expression]



""" Test that using starred nodes in unpacking
does not trigger a false positive on Python 3.
"""

__revision__ = 1

def test():
    """ Test that starred expressions don't give false positives. """
    first, second, *last = (1, 2, 3, 4)
    *last, = (1, 2)
    return (first, second, last)



"""Contains both normal error messages and Python3 porting error messages."""
# pylint: disable=too-few-public-methods

raise Exception, 1  # Error emitted here with the Python 3 checker.


class Test(object):
    """dummy"""

    def __init__(self):
        return 42
